
def agipd_module_number_from_mid_agipd_source(source):
    '''
    get the module number from a module source ame, e.g.
    input: MID_DET_AGIPD1M-1/DET/3CH0:xtdf
    output: 3
    '''
    moduleNumber = int(source.split("/")[-1].split(":")[0][:-3])
    return moduleNumber

def mid_agipd_source_from_agipd_module_number(module_number):
    '''
    get the source name from a module number, e.g.
    input: 3 
    output: MID_DET_AGIPD1M-1/DET/3CH0:xtdf
    '''
    
    source = f'MID_DET_AGIPD1M-1/DET/{module_number}3CH0:xtdf'
    
    return source

def agipd_module_name_from_mid_agipd_source(source):

    module_number = agipd_module_number_from_mid_agipd_source(source)
    module_name = agipd_module_name_from_agipd_module_number(module_number)
    
    return module_name

def mid_agipd_source_from_agipd_module_name(module_name):

    module_number = agipd_module_number_from_agipd_module_name(module_name)
    source = mid_agipd_source_from_agipd_module_number(module_number)
    
    return source

def agipd_module_name_from_agipd_module_number(module_number):
    match module_number:
        case 0:
            return "Q1M1"
        case 1:
            return "Q1M2"
        case 2:
            return "Q1M3"
        case 3:
            return "Q1M4"
        case 4:
            return "Q2M1"
        case 5:
            return "Q2M2"
        case 6:
            return "Q2M3"
        case 7:
            return "Q2M4"
        case 8:
            return "Q3M1"
        case 9:
            return "Q3M2"
        case 10:
            return "Q3M3"
        case 11:
            return "Q3M4"
        case 12:
            return "Q4M1"
        case 13:
            return "Q4M2"
        case 14:
            return "Q4M3"
        case 15:
            return "Q4M4"
        case _:
            print(f"ERROR in agipd_module_name_from_agipd_module_number, module number {module_number} is not a number from 0 to 15")
            return "noname"

def agipd_module_number_from_agipd_module_name(module_name):
    match module_name:
        case "Q1M1":
            return 0
        case "Q1M2":
            return 1
        case "Q1M3":
            return 2
        case "Q1M4":
            return 3
        case "Q2M1":
            return 4
        case "Q2M2":
            return 5
        case "Q2M3":
            return 6
        case "Q2M4":
            return 7
        case "Q3M1":
            return 8
        case "Q3M2":
            return 9
        case "Q3M3":
            return 10
        case "Q3M4":
            return 11
        case "Q4M1":
            return 12
        case "Q4M2":
            return 13
        case "Q4M3":
            return 14
        case "Q4M4":
            return 15
        case _:
            print(f"ERROR IN agipd_module_number_from_agipd_module_name, module_name {module_name} is not of the correct form")
            return -1


def generate_test_data(shape=None, q=0.02,photonEnergy=1, prob_below_threshold=0.005, prob_above_threshold=0.005, prob_nans=0.005):
    if shape is None:
        shape = (50, 512, 128)
    size = shape[0]*shape[1]*shape[2]

    propabilityValues = [q**0, q**1, q**2, q**3, q**4, q**5, q**6, prob_below_threshold/shape[0], prob_above_threshold/shape[0], prob_nans/shape[0]]
    values =            [   0,    1,    2,    3,    4,    5,    6, -40, 100, np.nan]
    photonData = np.reshape(
            np.random.choice(
                values, 
                size=size, 
                p=propabilityValues/np.sum(propabilityValues))
            ,shape)
    
    detectorData = (
        photonData
        *photonEnergy
        ).astype(np.float32)
    
    return detectorData
